# pylint: disable=too-many-lines
# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import sys
from typing import Any, AsyncIterable, Callable, Dict, IO, Optional, TypeVar, Union, overload
import urllib.parse

from azure.core.async_paging import AsyncItemPaged, AsyncList
from azure.core.exceptions import (
    ClientAuthenticationError,
    HttpResponseError,
    ResourceExistsError,
    ResourceNotFoundError,
    ResourceNotModifiedError,
    map_error,
)
from azure.core.pipeline import PipelineResponse
from azure.core.pipeline.transport import AsyncHttpResponse
from azure.core.rest import HttpRequest
from azure.core.tracing.decorator import distributed_trace
from azure.core.tracing.decorator_async import distributed_trace_async
from azure.core.utils import case_insensitive_dict
from azure.mgmt.core.exceptions import ARMErrorFormat

from ... import models as _models
from ..._vendor import _convert_request
from ...operations._products_operations import (
    build_get_product_request,
    build_get_products_request,
    build_get_request,
    build_list_details_request,
    build_list_products_request,
    build_list_request,
    build_upload_log_request,
)

if sys.version_info >= (3, 8):
    from typing import Literal  # pylint: disable=no-name-in-module, ungrouped-imports
else:
    from typing_extensions import Literal  # type: ignore  # pylint: disable=ungrouped-imports
T = TypeVar("T")
ClsType = Optional[Callable[[PipelineResponse[HttpRequest, AsyncHttpResponse], T, Dict[str, Any]], Any]]


class ProductsOperations:
    """
    .. warning::
        **DO NOT** instantiate this class directly.

        Instead, you should access the following operations through
        :class:`~azure.mgmt.azurestack.aio.AzureStackManagementClient`'s
        :attr:`products` attribute.
    """

    models = _models

    def __init__(self, *args, **kwargs) -> None:
        input_args = list(args)
        self._client = input_args.pop(0) if input_args else kwargs.pop("client")
        self._config = input_args.pop(0) if input_args else kwargs.pop("config")
        self._serialize = input_args.pop(0) if input_args else kwargs.pop("serializer")
        self._deserialize = input_args.pop(0) if input_args else kwargs.pop("deserializer")

    @distributed_trace
    def list(self, resource_group: str, registration_name: str, **kwargs: Any) -> AsyncIterable["_models.Product"]:
        """Returns a list of products.

        :param resource_group: Name of the resource group. Required.
        :type resource_group: str
        :param registration_name: Name of the Azure Stack registration. Required.
        :type registration_name: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: An iterator like instance of either Product or the result of cls(response)
        :rtype: ~azure.core.async_paging.AsyncItemPaged[~azure.mgmt.azurestack.models.Product]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: Literal["2022-06-01"] = kwargs.pop(
            "api_version", _params.pop("api-version", self._config.api_version)
        )
        cls: ClsType[_models.ProductList] = kwargs.pop("cls", None)

        error_map = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        def prepare_request(next_link=None):
            if not next_link:

                request = build_list_request(
                    resource_group=resource_group,
                    registration_name=registration_name,
                    subscription_id=self._config.subscription_id,
                    api_version=api_version,
                    template_url=self.list.metadata["url"],
                    headers=_headers,
                    params=_params,
                )
                request = _convert_request(request)
                request.url = self._client.format_url(request.url)

            else:
                # make call to next link with the client's api-version
                _parsed_next_link = urllib.parse.urlparse(next_link)
                _next_request_params = case_insensitive_dict(
                    {
                        key: [urllib.parse.quote(v) for v in value]
                        for key, value in urllib.parse.parse_qs(_parsed_next_link.query).items()
                    }
                )
                _next_request_params["api-version"] = self._config.api_version
                request = HttpRequest(
                    "GET", urllib.parse.urljoin(next_link, _parsed_next_link.path), params=_next_request_params
                )
                request = _convert_request(request)
                request.url = self._client.format_url(request.url)
                request.method = "GET"
            return request

        async def extract_data(pipeline_response):
            deserialized = self._deserialize("ProductList", pipeline_response)
            list_of_elem = deserialized.value
            if cls:
                list_of_elem = cls(list_of_elem)  # type: ignore
            return deserialized.next_link or None, AsyncList(list_of_elem)

        async def get_next(next_link=None):
            request = prepare_request(next_link)

            pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
                request, stream=False, **kwargs
            )
            response = pipeline_response.http_response

            if response.status_code not in [200]:
                map_error(status_code=response.status_code, response=response, error_map=error_map)
                error = self._deserialize.failsafe_deserialize(_models.ErrorResponse, pipeline_response)
                raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

            return pipeline_response

        return AsyncItemPaged(get_next, extract_data)

    list.metadata = {
        "url": "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroup}/providers/Microsoft.AzureStack/registrations/{registrationName}/products"
    }

    @distributed_trace_async
    async def get(
        self, resource_group: str, registration_name: str, product_name: str, **kwargs: Any
    ) -> _models.Product:
        """Returns the specified product.

        :param resource_group: Name of the resource group. Required.
        :type resource_group: str
        :param registration_name: Name of the Azure Stack registration. Required.
        :type registration_name: str
        :param product_name: Name of the product. Required.
        :type product_name: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: Product or the result of cls(response)
        :rtype: ~azure.mgmt.azurestack.models.Product
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: Literal["2022-06-01"] = kwargs.pop(
            "api_version", _params.pop("api-version", self._config.api_version)
        )
        cls: ClsType[_models.Product] = kwargs.pop("cls", None)

        request = build_get_request(
            resource_group=resource_group,
            registration_name=registration_name,
            product_name=product_name,
            subscription_id=self._config.subscription_id,
            api_version=api_version,
            template_url=self.get.metadata["url"],
            headers=_headers,
            params=_params,
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponse, pipeline_response)
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        deserialized = self._deserialize("Product", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    get.metadata = {
        "url": "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroup}/providers/Microsoft.AzureStack/registrations/{registrationName}/products/{productName}"
    }

    @distributed_trace_async
    async def list_details(
        self, resource_group: str, registration_name: str, product_name: str, **kwargs: Any
    ) -> _models.ExtendedProduct:
        """Returns the extended properties of a product.

        :param resource_group: Name of the resource group. Required.
        :type resource_group: str
        :param registration_name: Name of the Azure Stack registration. Required.
        :type registration_name: str
        :param product_name: Name of the product. Required.
        :type product_name: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: ExtendedProduct or the result of cls(response)
        :rtype: ~azure.mgmt.azurestack.models.ExtendedProduct
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: Literal["2022-06-01"] = kwargs.pop(
            "api_version", _params.pop("api-version", self._config.api_version)
        )
        cls: ClsType[_models.ExtendedProduct] = kwargs.pop("cls", None)

        request = build_list_details_request(
            resource_group=resource_group,
            registration_name=registration_name,
            product_name=product_name,
            subscription_id=self._config.subscription_id,
            api_version=api_version,
            template_url=self.list_details.metadata["url"],
            headers=_headers,
            params=_params,
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponse, pipeline_response)
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        deserialized = self._deserialize("ExtendedProduct", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    list_details.metadata = {
        "url": "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroup}/providers/Microsoft.AzureStack/registrations/{registrationName}/products/{productName}/listDetails"
    }

    @overload
    async def list_products(
        self,
        resource_group: str,
        registration_name: str,
        product_name: str,
        device_configuration: Optional[_models.DeviceConfiguration] = None,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.ProductList:
        """Returns a list of products.

        :param resource_group: Name of the resource group. Required.
        :type resource_group: str
        :param registration_name: Name of the Azure Stack registration. Required.
        :type registration_name: str
        :param product_name: Name of the product. Required.
        :type product_name: str
        :param device_configuration: Device configuration. Default value is None.
        :type device_configuration: ~azure.mgmt.azurestack.models.DeviceConfiguration
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: ProductList or the result of cls(response)
        :rtype: ~azure.mgmt.azurestack.models.ProductList
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    async def list_products(
        self,
        resource_group: str,
        registration_name: str,
        product_name: str,
        device_configuration: Optional[IO] = None,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.ProductList:
        """Returns a list of products.

        :param resource_group: Name of the resource group. Required.
        :type resource_group: str
        :param registration_name: Name of the Azure Stack registration. Required.
        :type registration_name: str
        :param product_name: Name of the product. Required.
        :type product_name: str
        :param device_configuration: Device configuration. Default value is None.
        :type device_configuration: IO
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Default value is "application/json".
        :paramtype content_type: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: ProductList or the result of cls(response)
        :rtype: ~azure.mgmt.azurestack.models.ProductList
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace_async
    async def list_products(
        self,
        resource_group: str,
        registration_name: str,
        product_name: str,
        device_configuration: Optional[Union[_models.DeviceConfiguration, IO]] = None,
        **kwargs: Any
    ) -> _models.ProductList:
        """Returns a list of products.

        :param resource_group: Name of the resource group. Required.
        :type resource_group: str
        :param registration_name: Name of the Azure Stack registration. Required.
        :type registration_name: str
        :param product_name: Name of the product. Required.
        :type product_name: str
        :param device_configuration: Device configuration. Is either a model type or a IO type. Default
         value is None.
        :type device_configuration: ~azure.mgmt.azurestack.models.DeviceConfiguration or IO
        :keyword content_type: Body Parameter content-type. Known values are: 'application/json'.
         Default value is None.
        :paramtype content_type: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: ProductList or the result of cls(response)
        :rtype: ~azure.mgmt.azurestack.models.ProductList
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: Literal["2022-06-01"] = kwargs.pop(
            "api_version", _params.pop("api-version", self._config.api_version)
        )
        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[_models.ProductList] = kwargs.pop("cls", None)

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(device_configuration, (IO, bytes)):
            _content = device_configuration
        else:
            if device_configuration is not None:
                _json = self._serialize.body(device_configuration, "DeviceConfiguration")
            else:
                _json = None

        request = build_list_products_request(
            resource_group=resource_group,
            registration_name=registration_name,
            product_name=product_name,
            subscription_id=self._config.subscription_id,
            api_version=api_version,
            content_type=content_type,
            json=_json,
            content=_content,
            template_url=self.list_products.metadata["url"],
            headers=_headers,
            params=_params,
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponse, pipeline_response)
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        deserialized = self._deserialize("ProductList", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    list_products.metadata = {
        "url": "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroup}/providers/Microsoft.AzureStack/registrations/{registrationName}/products/{productName}/listProducts"
    }

    @overload
    async def get_products(
        self,
        resource_group: str,
        registration_name: str,
        product_name: str,
        device_configuration: Optional[_models.DeviceConfiguration] = None,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.ProductList:
        """Returns a list of products.

        :param resource_group: Name of the resource group. Required.
        :type resource_group: str
        :param registration_name: Name of the Azure Stack registration. Required.
        :type registration_name: str
        :param product_name: Name of the product. Required.
        :type product_name: str
        :param device_configuration: Device configuration. Default value is None.
        :type device_configuration: ~azure.mgmt.azurestack.models.DeviceConfiguration
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: ProductList or the result of cls(response)
        :rtype: ~azure.mgmt.azurestack.models.ProductList
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    async def get_products(
        self,
        resource_group: str,
        registration_name: str,
        product_name: str,
        device_configuration: Optional[IO] = None,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.ProductList:
        """Returns a list of products.

        :param resource_group: Name of the resource group. Required.
        :type resource_group: str
        :param registration_name: Name of the Azure Stack registration. Required.
        :type registration_name: str
        :param product_name: Name of the product. Required.
        :type product_name: str
        :param device_configuration: Device configuration. Default value is None.
        :type device_configuration: IO
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Default value is "application/json".
        :paramtype content_type: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: ProductList or the result of cls(response)
        :rtype: ~azure.mgmt.azurestack.models.ProductList
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace_async
    async def get_products(
        self,
        resource_group: str,
        registration_name: str,
        product_name: str,
        device_configuration: Optional[Union[_models.DeviceConfiguration, IO]] = None,
        **kwargs: Any
    ) -> _models.ProductList:
        """Returns a list of products.

        :param resource_group: Name of the resource group. Required.
        :type resource_group: str
        :param registration_name: Name of the Azure Stack registration. Required.
        :type registration_name: str
        :param product_name: Name of the product. Required.
        :type product_name: str
        :param device_configuration: Device configuration. Is either a model type or a IO type. Default
         value is None.
        :type device_configuration: ~azure.mgmt.azurestack.models.DeviceConfiguration or IO
        :keyword content_type: Body Parameter content-type. Known values are: 'application/json'.
         Default value is None.
        :paramtype content_type: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: ProductList or the result of cls(response)
        :rtype: ~azure.mgmt.azurestack.models.ProductList
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: Literal["2022-06-01"] = kwargs.pop(
            "api_version", _params.pop("api-version", self._config.api_version)
        )
        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[_models.ProductList] = kwargs.pop("cls", None)

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(device_configuration, (IO, bytes)):
            _content = device_configuration
        else:
            if device_configuration is not None:
                _json = self._serialize.body(device_configuration, "DeviceConfiguration")
            else:
                _json = None

        request = build_get_products_request(
            resource_group=resource_group,
            registration_name=registration_name,
            product_name=product_name,
            subscription_id=self._config.subscription_id,
            api_version=api_version,
            content_type=content_type,
            json=_json,
            content=_content,
            template_url=self.get_products.metadata["url"],
            headers=_headers,
            params=_params,
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponse, pipeline_response)
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        deserialized = self._deserialize("ProductList", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    get_products.metadata = {
        "url": "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroup}/providers/Microsoft.AzureStack/registrations/{registrationName}/products/{productName}/getProducts"
    }

    @overload
    async def get_product(
        self,
        resource_group: str,
        registration_name: str,
        product_name: str,
        device_configuration: Optional[_models.DeviceConfiguration] = None,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.Product:
        """Returns the specified product.

        :param resource_group: Name of the resource group. Required.
        :type resource_group: str
        :param registration_name: Name of the Azure Stack registration. Required.
        :type registration_name: str
        :param product_name: Name of the product. Required.
        :type product_name: str
        :param device_configuration: Device configuration. Default value is None.
        :type device_configuration: ~azure.mgmt.azurestack.models.DeviceConfiguration
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: Product or the result of cls(response)
        :rtype: ~azure.mgmt.azurestack.models.Product
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    async def get_product(
        self,
        resource_group: str,
        registration_name: str,
        product_name: str,
        device_configuration: Optional[IO] = None,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.Product:
        """Returns the specified product.

        :param resource_group: Name of the resource group. Required.
        :type resource_group: str
        :param registration_name: Name of the Azure Stack registration. Required.
        :type registration_name: str
        :param product_name: Name of the product. Required.
        :type product_name: str
        :param device_configuration: Device configuration. Default value is None.
        :type device_configuration: IO
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Default value is "application/json".
        :paramtype content_type: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: Product or the result of cls(response)
        :rtype: ~azure.mgmt.azurestack.models.Product
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace_async
    async def get_product(
        self,
        resource_group: str,
        registration_name: str,
        product_name: str,
        device_configuration: Optional[Union[_models.DeviceConfiguration, IO]] = None,
        **kwargs: Any
    ) -> _models.Product:
        """Returns the specified product.

        :param resource_group: Name of the resource group. Required.
        :type resource_group: str
        :param registration_name: Name of the Azure Stack registration. Required.
        :type registration_name: str
        :param product_name: Name of the product. Required.
        :type product_name: str
        :param device_configuration: Device configuration. Is either a model type or a IO type. Default
         value is None.
        :type device_configuration: ~azure.mgmt.azurestack.models.DeviceConfiguration or IO
        :keyword content_type: Body Parameter content-type. Known values are: 'application/json'.
         Default value is None.
        :paramtype content_type: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: Product or the result of cls(response)
        :rtype: ~azure.mgmt.azurestack.models.Product
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: Literal["2022-06-01"] = kwargs.pop(
            "api_version", _params.pop("api-version", self._config.api_version)
        )
        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[_models.Product] = kwargs.pop("cls", None)

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(device_configuration, (IO, bytes)):
            _content = device_configuration
        else:
            if device_configuration is not None:
                _json = self._serialize.body(device_configuration, "DeviceConfiguration")
            else:
                _json = None

        request = build_get_product_request(
            resource_group=resource_group,
            registration_name=registration_name,
            product_name=product_name,
            subscription_id=self._config.subscription_id,
            api_version=api_version,
            content_type=content_type,
            json=_json,
            content=_content,
            template_url=self.get_product.metadata["url"],
            headers=_headers,
            params=_params,
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponse, pipeline_response)
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        deserialized = self._deserialize("Product", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    get_product.metadata = {
        "url": "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroup}/providers/Microsoft.AzureStack/registrations/{registrationName}/products/{productName}/getProduct"
    }

    @overload
    async def upload_log(
        self,
        resource_group: str,
        registration_name: str,
        product_name: str,
        marketplace_product_log_update: Optional[_models.MarketplaceProductLogUpdate] = None,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.ProductLog:
        """Returns the specified product.

        :param resource_group: Name of the resource group. Required.
        :type resource_group: str
        :param registration_name: Name of the Azure Stack registration. Required.
        :type registration_name: str
        :param product_name: Name of the product. Required.
        :type product_name: str
        :param marketplace_product_log_update: Update details for product log. Default value is None.
        :type marketplace_product_log_update: ~azure.mgmt.azurestack.models.MarketplaceProductLogUpdate
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: ProductLog or the result of cls(response)
        :rtype: ~azure.mgmt.azurestack.models.ProductLog
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    async def upload_log(
        self,
        resource_group: str,
        registration_name: str,
        product_name: str,
        marketplace_product_log_update: Optional[IO] = None,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.ProductLog:
        """Returns the specified product.

        :param resource_group: Name of the resource group. Required.
        :type resource_group: str
        :param registration_name: Name of the Azure Stack registration. Required.
        :type registration_name: str
        :param product_name: Name of the product. Required.
        :type product_name: str
        :param marketplace_product_log_update: Update details for product log. Default value is None.
        :type marketplace_product_log_update: IO
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Default value is "application/json".
        :paramtype content_type: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: ProductLog or the result of cls(response)
        :rtype: ~azure.mgmt.azurestack.models.ProductLog
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace_async
    async def upload_log(
        self,
        resource_group: str,
        registration_name: str,
        product_name: str,
        marketplace_product_log_update: Optional[Union[_models.MarketplaceProductLogUpdate, IO]] = None,
        **kwargs: Any
    ) -> _models.ProductLog:
        """Returns the specified product.

        :param resource_group: Name of the resource group. Required.
        :type resource_group: str
        :param registration_name: Name of the Azure Stack registration. Required.
        :type registration_name: str
        :param product_name: Name of the product. Required.
        :type product_name: str
        :param marketplace_product_log_update: Update details for product log. Is either a model type
         or a IO type. Default value is None.
        :type marketplace_product_log_update: ~azure.mgmt.azurestack.models.MarketplaceProductLogUpdate
         or IO
        :keyword content_type: Body Parameter content-type. Known values are: 'application/json'.
         Default value is None.
        :paramtype content_type: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: ProductLog or the result of cls(response)
        :rtype: ~azure.mgmt.azurestack.models.ProductLog
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: Literal["2022-06-01"] = kwargs.pop(
            "api_version", _params.pop("api-version", self._config.api_version)
        )
        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[_models.ProductLog] = kwargs.pop("cls", None)

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(marketplace_product_log_update, (IO, bytes)):
            _content = marketplace_product_log_update
        else:
            if marketplace_product_log_update is not None:
                _json = self._serialize.body(marketplace_product_log_update, "MarketplaceProductLogUpdate")
            else:
                _json = None

        request = build_upload_log_request(
            resource_group=resource_group,
            registration_name=registration_name,
            product_name=product_name,
            subscription_id=self._config.subscription_id,
            api_version=api_version,
            content_type=content_type,
            json=_json,
            content=_content,
            template_url=self.upload_log.metadata["url"],
            headers=_headers,
            params=_params,
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponse, pipeline_response)
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        deserialized = self._deserialize("ProductLog", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    upload_log.metadata = {
        "url": "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroup}/providers/Microsoft.AzureStack/registrations/{registrationName}/products/{productName}/uploadProductLog"
    }



# Copyright (c) Microsoft Corporation.
# Licensed under the MIT license.
"""Tests for validator generator contract behavior."""

from __future__ import annotations

import subprocess
import sys
from pathlib import Path


def _script_path() -> Path:
    return Path(__file__).resolve().parents[2] / "scripts" / "generate_validators.py"


def _minimal_spec() -> str:
    return """{
  "paths": {
    "/responses": {
      "post": {
        "requestBody": {
          "content": {
            "application/json": {
              "schema": {
                "$ref": "#/components/schemas/CreateResponse"
              }
            }
          }
        }
      }
    }
  },
  "components": {
    "schemas": {
      "CreateResponse": {
        "type": "object",
        "required": ["model"],
        "properties": {
          "model": {"type": "string"}
        }
      }
    }
  }
}
"""


def test_generator_requires_cli_args() -> None:
    proc = subprocess.run(
        [sys.executable, str(_script_path())],
        capture_output=True,
        text=True,
        check=False,
    )
    assert proc.returncode != 0
    assert "--input" in proc.stderr
    assert "--output" in proc.stderr


def test_generated_file_has_autogen_header(tmp_path: Path) -> None:
    spec_path = tmp_path / "spec.json"
    out_path = tmp_path / "_validators.py"
    spec_path.write_text(_minimal_spec(), encoding="utf-8")

    proc = subprocess.run(
        [
            sys.executable,
            str(_script_path()),
            "--input",
            str(spec_path),
            "--output",
            str(out_path),
            "--root-schemas",
            "CreateResponse",
        ],
        capture_output=True,
        text=True,
        check=False,
    )

    assert proc.returncode == 0, proc.stderr
    content = out_path.read_text(encoding="utf-8")
    assert content.startswith("# pylint: disable=line-too-long,useless-suppression,too-many-lines")
    assert "# Code generated by Microsoft (R) Python Code Generator." in content


def test_generation_is_deterministic_for_same_input(tmp_path: Path) -> None:
    spec_path = tmp_path / "spec.json"
    out_path = tmp_path / "_validators.py"
    spec_path.write_text(_minimal_spec(), encoding="utf-8")

    cmd = [
        sys.executable,
        str(_script_path()),
        "--input",
        str(spec_path),
        "--output",
        str(out_path),
        "--root-schemas",
        "CreateResponse",
    ]

    first = subprocess.run(cmd, capture_output=True, text=True, check=False)
    assert first.returncode == 0, first.stderr
    first_output = out_path.read_text(encoding="utf-8")

    second = subprocess.run(cmd, capture_output=True, text=True, check=False)
    assert second.returncode == 0, second.stderr
    second_output = out_path.read_text(encoding="utf-8")

    assert first_output == second_output


def test_overlay_removes_required_field(tmp_path: Path) -> None:
    """--overlay required:[] removes all required checks from the named schema."""
    spec_path = tmp_path / "spec.json"
    out_path = tmp_path / "_validators.py"
    overlay_path = tmp_path / "overlay.yaml"

    spec_path.write_text(_minimal_spec(), encoding="utf-8")
    # overlay removes model from required
    overlay_path.write_text("schemas:\n  CreateResponse:\n    required: []\n", encoding="utf-8")

    proc = subprocess.run(
        [
            sys.executable,
            str(_script_path()),
            "--input",
            str(spec_path),
            "--output",
            str(out_path),
            "--root-schemas",
            "CreateResponse",
            "--overlay",
            str(overlay_path),
        ],
        capture_output=True,
        text=True,
        check=False,
    )
    assert proc.returncode == 0, proc.stderr
    content = out_path.read_text(encoding="utf-8")
    # "Required property 'model'" must not appear when required is cleared
    assert "Required property 'model'" not in content


def test_overlay_not_required_marks_field_nullable(tmp_path: Path) -> None:
    """--overlay not_required removes a field from required and adds a None guard."""
    spec_path = tmp_path / "spec.json"
    out_path = tmp_path / "_validators.py"
    overlay_path = tmp_path / "overlay.yaml"

    spec_path.write_text(
        """{
  "paths": {},
  "components": {
    "schemas": {
      "Item": {
        "type": "object",
        "required": ["type", "role"],
        "properties": {
          "type": {"type": "string"},
          "role": {"type": "string"}
        }
      }
    }
  }
}""",
        encoding="utf-8",
    )
    overlay_path.write_text("schemas:\n  Item:\n    not_required:\n      - type\n", encoding="utf-8")

    proc = subprocess.run(
        [
            sys.executable,
            str(_script_path()),
            "--input",
            str(spec_path),
            "--output",
            str(out_path),
            "--root-schemas",
            "Item",
            "--overlay",
            str(overlay_path),
        ],
        capture_output=True,
        text=True,
        check=False,
    )
    assert proc.returncode == 0, proc.stderr
    content = out_path.read_text(encoding="utf-8")
    # 'type' must NOT appear as a required property check
    assert "Required property 'type'" not in content
    # 'role' must still be required
    assert "Required property 'role'" in content



# Copyright (c) Microsoft Corporation.
# Licensed under the MIT license.
"""Contract tests for distributed tracing on POST /responses.

These tests verify the span display name, GenAI parity tags, additional OTEL
tags, request ID propagation, and baggage items emitted by ``handle_create``.
"""

from __future__ import annotations

from typing import Any

from starlette.testclient import TestClient

from azure.ai.agentserver.responses import ResponsesAgentServerHost, ResponsesServerOptions
from azure.ai.agentserver.responses.hosting._observability import InMemoryCreateSpanHook


def _noop_handler(request: Any, context: Any, cancellation_signal: Any):
    async def _events():
        if False:  # pragma: no cover
            yield None

    return _events()


def _build_client(hook: InMemoryCreateSpanHook | None = None) -> TestClient:
    options = ResponsesServerOptions(create_span_hook=hook)
    app = ResponsesAgentServerHost(options=options)
    app.response_handler(_noop_handler)
    return TestClient(app)


# ---------------------------------------------------------------------------
# Span display name
# ---------------------------------------------------------------------------


def test_tracing__span_name_includes_model() -> None:
    hook = InMemoryCreateSpanHook()
    client = _build_client(hook)

    client.post(
        "/responses",
        json={"model": "gpt-4o-mini", "input": "hi", "stream": False},
    )

    assert len(hook.spans) == 1
    assert hook.spans[0].name == "create_response"


def test_tracing__span_name_without_model_falls_back_to_create_response() -> None:
    """When model is absent the span name should still be emitted."""
    hook = InMemoryCreateSpanHook()
    client = _build_client(hook)

    client.post(
        "/responses",
        json={"input": "hi", "stream": False},
    )

    # Span must be recorded regardless.
    assert len(hook.spans) == 1
    assert hook.spans[0].name == "create_response"


# ---------------------------------------------------------------------------
# GenAI parity tags
# ---------------------------------------------------------------------------


def test_tracing__span_tags_include_genai_parity_fields() -> None:
    hook = InMemoryCreateSpanHook()
    client = _build_client(hook)

    client.post(
        "/responses",
        json={
            "model": "gpt-4o",
            "input": "hello",
            "stream": False,
            "agent_reference": {"type": "agent_reference", "name": "my-agent", "version": "v2"},
        },
    )

    assert len(hook.spans) == 1
    tags = hook.spans[0].tags

    assert isinstance(tags.get("gen_ai.response.id"), str)
    assert tags["gen_ai.response.id"].startswith("caresp_") or tags["gen_ai.response.id"].startswith("resp_")
    assert tags["gen_ai.agent.name"] == "my-agent"
    assert tags["gen_ai.agent.id"] == "my-agent:v2"
    assert tags["gen_ai.provider.name"] == "AzureAI Hosted Agents"
    assert tags["service.name"] == "azure.ai.agentserver"


# ---------------------------------------------------------------------------
# Additional OTEL tags
# ---------------------------------------------------------------------------


def test_tracing__span_tags_include_operation_name_invoke_agent() -> None:
    hook = InMemoryCreateSpanHook()
    client = _build_client(hook)

    client.post(
        "/responses",
        json={"model": "gpt-4o-mini", "input": "hi", "stream": False},
    )

    assert hook.spans[0].tags["gen_ai.operation.name"] == "invoke_agent"


def test_tracing__span_tags_include_model() -> None:
    hook = InMemoryCreateSpanHook()
    client = _build_client(hook)

    client.post(
        "/responses",
        json={"model": "gpt-4o-mini", "input": "hi", "stream": False},
    )

    assert hook.spans[0].tags["gen_ai.request.model"] == "gpt-4o-mini"


def test_tracing__span_tags_include_conversation_id_when_present() -> None:
    hook = InMemoryCreateSpanHook()
    client = _build_client(hook)

    client.post(
        "/responses",
        json={
            "model": "gpt-4o-mini",
            "input": "hi",
            "stream": False,
            "conversation": "conv_abc123",
        },
    )

    assert hook.spans[0].tags.get("gen_ai.conversation.id") == "conv_abc123"


def test_tracing__span_tags_omit_conversation_id_when_absent() -> None:
    hook = InMemoryCreateSpanHook()
    client = _build_client(hook)

    client.post(
        "/responses",
        json={"model": "gpt-4o-mini", "input": "hi", "stream": False},
    )

    assert "gen_ai.conversation.id" not in hook.spans[0].tags


def test_tracing__span_tags_include_agent_version() -> None:
    hook = InMemoryCreateSpanHook()
    client = _build_client(hook)

    client.post(
        "/responses",
        json={
            "model": "gpt-4o-mini",
            "input": "hi",
            "stream": False,
            "agent_reference": {"type": "agent_reference", "name": "bot", "version": "1.0"},
        },
    )

    assert hook.spans[0].tags.get("gen_ai.agent.version") == "1.0"


# ---------------------------------------------------------------------------
# Request ID propagation
# ---------------------------------------------------------------------------


def test_tracing__span_tags_include_request_id_from_header() -> None:
    hook = InMemoryCreateSpanHook()
    client = _build_client(hook)

    client.post(
        "/responses",
        json={"model": "gpt-4o-mini", "input": "hi", "stream": False},
        headers={"X-Request-Id": "req-abc-123"},
    )

    assert hook.spans[0].tags.get("request.id") == "req-abc-123"


def test_tracing__request_id_truncated_to_256_characters() -> None:
    hook = InMemoryCreateSpanHook()
    client = _build_client(hook)

    long_id = "x" * 300

    client.post(
        "/responses",
        json={"model": "gpt-4o-mini", "input": "hi", "stream": False},
        headers={"X-Request-Id": long_id},
    )

    value = hook.spans[0].tags.get("request.id")
    assert value is not None
    assert len(value) == 256
    assert value == "x" * 256


def test_tracing__span_tags_omit_request_id_when_header_absent() -> None:
    hook = InMemoryCreateSpanHook()
    client = _build_client(hook)

    client.post(
        "/responses",
        json={"model": "gpt-4o-mini", "input": "hi", "stream": False},
    )

    assert "request.id" not in hook.spans[0].tags



# pylint: disable=line-too-long,useless-suppression
import sys
import logging
import functools
import json
import datetime
import os
import base64
from devtools_testutils import (
    AzureRecordedTestCase,
    EnvironmentVariableLoader,
)
from devtools_testutils.azure_testcase import is_live
from azure.ai.agents.models import (
    MessageTextFileCitationDetails,
    MessageTextUrlCitationDetails,
    RunStepBrowserAutomationToolCall,
    ThreadMessage,
)

agentClientPreparer = functools.partial(
    EnvironmentVariableLoader,
    "azure_ai_agents",
    # TODO: uncomment this endpoint when re running with 1DP
    # azure_ai_agents_tests_project_endpoint="https://aiservices-id.services.ai.azure.com/api/projects/project-name",
    # TODO: remove this endpoint when re running with 1DP
    azure_ai_agents_tests_project_connection_string="https://Sanitized.api.azureml.ms/agents/v1.0/subscriptions/00000000-0000-0000-0000-000000000000/resourceGroups/00000/providers/Microsoft.MachineLearningServices/workspaces/00000/",
    azure_ai_agents_tests_project_endpoint="https://Sanitized.services.ai.azure.com/api/projects/00000",
    azure_ai_agents_tests_data_path="azureml://subscriptions/00000000-0000-0000-0000-000000000000/resourcegroups/rg-resour-cegr-oupfoo1/workspaces/abcd-abcdabcdabcda-abcdefghijklm/datastores/workspaceblobstore/paths/LocalUpload/000000000000/product_info_1.md",
    azure_ai_agents_tests_storage_queue="https://foobar.queue.core.windows.net",
    azure_ai_agents_tests_search_index_name="sample_index",
    azure_ai_agents_tests_search_connection_id="/subscriptions/00000000-0000-0000-0000-000000000000/resourceGroups/00000/providers/Microsoft.MachineLearningServices/workspaces/00000/connections/someindex",
    azure_ai_agents_tests_bing_connection_id="/subscriptions/00000000-0000-0000-0000-000000000000/resourceGroups/00000/providers/Microsoft.CognitiveServices/accounts/00000/projects/00000/connections/00000",
    azure_ai_agents_tests_playwright_connection_id="/subscriptions/00000000-0000-0000-0000-000000000000/resourceGroups/00000/providers/Microsoft.CognitiveServices/accounts/00000/projects/00000/connections/00000",
    azure_ai_agents_tests_deep_research_model="gpt-4o-deep-research",
    azure_ai_agents_tests_is_test_run="True",
    azure_ai_agents_tests_bing_custom_connection_id="/subscriptions/00000000-0000-0000-0000-000000000000/resourceGroups/00000/providers/Microsoft.CognitiveServices/accounts/00000/projects/00000/connections/00000",
    azure_ai_agents_tests_bing_configuration_name="sample_configuration",
    azure_ai_agents_tests_fabric_connection_id="/subscriptions/00000000-0000-0000-0000-000000000000/resourceGroups/00000/providers/Microsoft.CognitiveServices/accounts/00000/projects/00000/connections/00000",
    azure_ai_agents_tests_sharepoint_connection_id="/subscriptions/00000000-0000-0000-0000-000000000000/resourceGroups/00000/providers/Microsoft.CognitiveServices/accounts/00000/projects/00000/connections/00000",
)

# Set to True to enable SDK logging
LOGGING_ENABLED = True

if LOGGING_ENABLED:
    # Create a logger for the 'azure' SDK
    # See https://docs.python.org/3/library/logging.html
    logger = logging.getLogger("azure")
    logger.setLevel(logging.DEBUG)  # INFO or DEBUG

    # Configure a console output
    handler = logging.StreamHandler(stream=sys.stdout)
    logger.addHandler(handler)


def fetch_current_datetime_live():
    """
    Get the current time as a JSON string.

    :return: Static time string so that test recordings work.
    :rtype: str
    """
    current_datetime = datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")
    time_json = json.dumps({"current_time": current_datetime})
    return time_json


def fetch_current_datetime_recordings():
    """
    Get the current time as a JSON string.

    :return: Static time string so that test recordings work.
    :rtype: str
    """
    time_json = json.dumps({"current_time": "2024-10-10 12:30:19"})
    return time_json


def image_to_base64(image_path: str) -> str:
    """
    Convert an image file to a Base64-encoded string.

    :param image_path: The path to the image file (e.g. 'image_file.png')
    :return: A Base64-encoded string representing the image.
    :raises FileNotFoundError: If the provided file path does not exist.
    :raises OSError: If there's an error reading the file.
    """
    if not os.path.isfile(image_path):
        raise FileNotFoundError(f"File not found at: {image_path}")

    try:
        with open(image_path, "rb") as image_file:
            file_data = image_file.read()
        return base64.b64encode(file_data).decode("utf-8")
    except Exception as exc:
        raise OSError(f"Error reading file '{image_path}'") from exc


class TestAgentClientBase(AzureRecordedTestCase):
    """Base class for Agents Client tests. Please put all code common to sync and async tests here."""

    @classmethod
    def _sleep_time(cls, sleep: int = 1) -> int:
        """Return sleep or zero if we are running the recording."""
        return sleep if is_live() else 0

    @classmethod
    def _has_url_annotation(cls, message: ThreadMessage, uri_annotation: MessageTextUrlCitationDetails) -> bool:
        """
        Return True if the message contains required URL annotation.

        :param message: The message to look for annotations.
        :param uri_annotation: The annotation to look for.
        :return:  True if the message contains the required URL annotation.
        :rtype: bool
        """
        url_annotations = message.url_citation_annotations
        if url_annotations:
            for url in url_annotations:
                if (
                    (uri_annotation.url == "*" and url.url_citation.url) or url.url_citation.url == uri_annotation.url
                ) and (
                    (uri_annotation.title == "*" and url.url_citation.title)
                    or url.url_citation.title == uri_annotation.title
                ):
                    return True
        return False

    @classmethod
    def _has_file_annotation(cls, message: ThreadMessage, file_annotation: MessageTextFileCitationDetails) -> bool:
        """
        Return True if the message contains required file annotation

        :param message: The message to look for annotations.
        :param file_annotation: The annotation to look for.
        :return: True if the message contains the required file annotation, otherwise False.
        :rtype: bool
        """
        file_annotations = message.file_citation_annotations
        if file_annotations:
            for fle in file_annotations:
                if fle.file_citation:
                    if fle.file_citation.file_id == file_annotation.file_citation.file_id:
                        return True
        return False

    @classmethod
    def _validate_run_step_browser_automation_tool_call(cls, tool_call: RunStepBrowserAutomationToolCall):
        assert tool_call.browser_automation.input
        assert tool_call.browser_automation.output
        assert len(tool_call.browser_automation.steps) > 1
        assert tool_call.browser_automation.steps[0].last_step_result
        assert tool_call.browser_automation.steps[0].current_state
        assert tool_call.browser_automation.steps[0].next_step



# coding=utf-8
# pylint: disable=wrong-import-position

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from ._patch import *  # pylint: disable=unused-wildcard-import

from ._operations import BooleanOperations  # type: ignore
from ._operations import StringOperations  # type: ignore
from ._operations import BytesOperations  # type: ignore
from ._operations import IntOperations  # type: ignore
from ._operations import FloatOperations  # type: ignore
from ._operations import DecimalOperations  # type: ignore
from ._operations import Decimal128Operations  # type: ignore
from ._operations import DatetimeOperations  # type: ignore
from ._operations import DurationOperations  # type: ignore
from ._operations import EnumOperations  # type: ignore
from ._operations import ExtensibleEnumOperations  # type: ignore
from ._operations import ModelOperations  # type: ignore
from ._operations import CollectionsStringOperations  # type: ignore
from ._operations import CollectionsIntOperations  # type: ignore
from ._operations import CollectionsModelOperations  # type: ignore
from ._operations import DictionaryStringOperations  # type: ignore
from ._operations import NeverOperations  # type: ignore
from ._operations import UnknownStringOperations  # type: ignore
from ._operations import UnknownIntOperations  # type: ignore
from ._operations import UnknownDictOperations  # type: ignore
from ._operations import UnknownArrayOperations  # type: ignore
from ._operations import StringLiteralOperations  # type: ignore
from ._operations import IntLiteralOperations  # type: ignore
from ._operations import FloatLiteralOperations  # type: ignore
from ._operations import BooleanLiteralOperations  # type: ignore
from ._operations import UnionStringLiteralOperations  # type: ignore
from ._operations import UnionIntLiteralOperations  # type: ignore
from ._operations import UnionFloatLiteralOperations  # type: ignore
from ._operations import UnionEnumValueOperations  # type: ignore

from ._patch import __all__ as _patch_all
from ._patch import *
from ._patch import patch_sdk as _patch_sdk

__all__ = [
    "BooleanOperations",
    "StringOperations",
    "BytesOperations",
    "IntOperations",
    "FloatOperations",
    "DecimalOperations",
    "Decimal128Operations",
    "DatetimeOperations",
    "DurationOperations",
    "EnumOperations",
    "ExtensibleEnumOperations",
    "ModelOperations",
    "CollectionsStringOperations",
    "CollectionsIntOperations",
    "CollectionsModelOperations",
    "DictionaryStringOperations",
    "NeverOperations",
    "UnknownStringOperations",
    "UnknownIntOperations",
    "UnknownDictOperations",
    "UnknownArrayOperations",
    "StringLiteralOperations",
    "IntLiteralOperations",
    "FloatLiteralOperations",
    "BooleanLiteralOperations",
    "UnionStringLiteralOperations",
    "UnionIntLiteralOperations",
    "UnionFloatLiteralOperations",
    "UnionEnumValueOperations",
]
__all__.extend([p for p in _patch_all if p not in __all__])  # pyright: ignore
_patch_sdk()



# coding=utf-8
# pylint: disable=useless-super-delegation

import datetime
from typing import Any, Mapping, overload

from .._utils.model_base import Model as _Model, rest_field


class DefaultDatetimeProperty(_Model):
    """DefaultDatetimeProperty.

    :ivar value: Required.
    :vartype value: ~datetime.datetime
    """

    value: datetime.datetime = rest_field(visibility=["read", "create", "update", "delete", "query"], format="rfc3339")
    """Required."""

    @overload
    def __init__(
        self,
        *,
        value: datetime.datetime,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class Rfc3339DatetimeProperty(_Model):
    """Rfc3339DatetimeProperty.

    :ivar value: Required.
    :vartype value: ~datetime.datetime
    """

    value: datetime.datetime = rest_field(visibility=["read", "create", "update", "delete", "query"], format="rfc3339")
    """Required."""

    @overload
    def __init__(
        self,
        *,
        value: datetime.datetime,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class Rfc7231DatetimeProperty(_Model):
    """Rfc7231DatetimeProperty.

    :ivar value: Required.
    :vartype value: ~datetime.datetime
    """

    value: datetime.datetime = rest_field(visibility=["read", "create", "update", "delete", "query"], format="rfc7231")
    """Required."""

    @overload
    def __init__(
        self,
        *,
        value: datetime.datetime,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class UnixTimestampArrayDatetimeProperty(_Model):
    """UnixTimestampArrayDatetimeProperty.

    :ivar value: Required.
    :vartype value: list[~datetime.datetime]
    """

    value: list[datetime.datetime] = rest_field(
        visibility=["read", "create", "update", "delete", "query"], format="unix-timestamp"
    )
    """Required."""

    @overload
    def __init__(
        self,
        *,
        value: list[datetime.datetime],
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class UnixTimestampDatetimeProperty(_Model):
    """UnixTimestampDatetimeProperty.

    :ivar value: Required.
    :vartype value: ~datetime.datetime
    """

    value: datetime.datetime = rest_field(
        visibility=["read", "create", "update", "delete", "query"], format="unix-timestamp"
    )
    """Required."""

    @overload
    def __init__(
        self,
        *,
        value: datetime.datetime,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import pytest
from devtools_testutils import recorded_by_proxy
from testpreparer import IndividuallyParentClientTestBase, IndividuallyParentPreparer


@pytest.mark.skip("you may need to update the auto-generated test case before run it")
class TestIndividuallyParentIndividuallyParentNestedWithMixedClientOperations(IndividuallyParentClientTestBase):
    @IndividuallyParentPreparer()
    @recorded_by_proxy
    def test_individually_parent_nested_with_mixed_client_with_query(self, individuallyparent_endpoint):
        client = self.create_client(endpoint=individuallyparent_endpoint)
        response = client.individually_parent_nested_with_mixed_client.with_query(
            region="str",
        )

        # please add some check logic here by yourself
        # ...

    @IndividuallyParentPreparer()
    @recorded_by_proxy
    def test_individually_parent_nested_with_mixed_client_get_standalone(self, individuallyparent_endpoint):
        client = self.create_client(endpoint=individuallyparent_endpoint)
        response = client.individually_parent_nested_with_mixed_client.get_standalone(
            region="str",
        )

        # please add some check logic here by yourself
        # ...

    @IndividuallyParentPreparer()
    @recorded_by_proxy
    def test_individually_parent_nested_with_mixed_client_delete_standalone(self, individuallyparent_endpoint):
        client = self.create_client(endpoint=individuallyparent_endpoint)
        response = client.individually_parent_nested_with_mixed_client.delete_standalone(
            region="str",
        )

        # please add some check logic here by yourself
        # ...



# pylint: disable=line-too-long,useless-suppression
import functools
import pytest

from devtools_testutils import AzureRecordedTestCase, EnvironmentVariableLoader
from devtools_testutils.aio import recorded_by_proxy_async
from azure.core.credentials import AzureKeyCredential
from azure.ai.textanalytics.authoring.aio import TextAuthoringClient
from azure.ai.textanalytics.authoring.models import (
    CustomSingleLabelClassificationDocumentEvalResult,
)

AuthoringPreparer = functools.partial(
    EnvironmentVariableLoader,
    "authoring",
    authoring_endpoint="https://Sanitized.cognitiveservices.azure.com/",
    authoring_key="fake_key",
)


class TestTextAuthoring(AzureRecordedTestCase):
    pass


class TestTextAuthoringGetModelEvaluationResultsAsync(TestTextAuthoring):
    @AuthoringPreparer()
    @recorded_by_proxy_async
    @pytest.mark.asyncio
    async def test_get_model_evaluation_results_async(self, authoring_endpoint, authoring_key):
        async with TextAuthoringClient(authoring_endpoint, AzureKeyCredential(authoring_key)) as client:
            project_name = "single-class-project"
            trained_model_label = "model3"

            # Trained-model–scoped call
            project_client = client.get_project_client(project_name)
            results = project_client.trained_model.list_model_evaluation_results(
                trained_model_label,
                string_index_type="UTF16CodeUnit",
            )

            # Assert the pager exists
            assert results is not None, "The evaluation results should not be null."

            async for result in results:
                # Base validations
                assert result is not None, "The result should not be null."
                assert result.location is not None, "The result location should not be null."
                assert result.language is not None, "The result language should not be null."

                # Validate classification result
                if isinstance(result, CustomSingleLabelClassificationDocumentEvalResult):
                    classification = result.custom_single_label_classification_result
                    assert classification is not None, "The classification result should not be null."
                    assert (
                        classification.expected_class and classification.expected_class.strip()
                    ), "The expected class should not be null or empty."
                    assert (
                        classification.predicted_class and classification.predicted_class.strip()
                    ), "The predicted class should not be null or empty."

                    # Optional: print a couple of fields for recording visibility
                    print(f"Document Location: {result.location}")
                    print("  Classification:")
                    print(f"    Expected: {classification.expected_class}")
                    print(f"    Predicted: {classification.predicted_class}")
                else:
                    raise AssertionError(f"Unsupported result type: {type(result).__name__}")



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from ._batch_perf_test import BatchPerfTest


class PerfStressTest(BatchPerfTest):

    def run_batch_sync(self) -> int:
        self.run_sync()
        return 1

    async def run_batch_async(self) -> int:
        await self.run_async()
        return 1

    def run_sync(self) -> None:
        raise NotImplementedError("run_sync must be implemented for {}".format(self.__class__.__name__))

    async def run_async(self) -> None:
        raise NotImplementedError("run_async must be implemented for {}".format(self.__class__.__name__))
